@@ -95,6 +95,83 @@ sub _parse_def_modifiers {
 }
 
 
+sub optimise {
+  my ($self) = @_;
+
+  my $commands = $self->{compiled_commands};
+
+
+
+  ## Internal cat pass
+
+  my @tmp_commands;
+  my $internal_cats_removed = 0;
+
+  foreach my $command (@$commands) {
+    if ($command->{shell_arg}->[0] eq 'cat' &&
+        @{$command->{shell_arg}} == 1 &&
+        $command != $commands->[0] &&
+        $command != $commands->[-1]) {
+      $internal_cats_removed++;
+      next;
+    }
+
+    push @tmp_commands, $command;
+  }
+
+  if ($internal_cats_removed) {
+    debug_log("OPT: removed $internal_cats_removed internal cats");
+    $commands = \@tmp_commands; 
+  }
+
+
+
+
+  ## Leading cat pass
+
+  my $first_command = $commands->[0]->{shell_arg};
+
+  if ($first_command->[0] eq 'cat') {
+    if (@$first_command == 1) {
+      debug_log("OPT: optimised away empty leading cat");
+      shift @$commands;
+    }
+    if (@$first_command == 2) {
+      my $file = $first_command->[1];
+      debug_log("OPT: optimised away leading cat, opening file '$file' as STDIN");
+      if (!open(STDIN, '<', $file)) {
+        say STDERR "cat: $file: No such file or directory";
+        exit(1);
+      }
+      shift @$commands;
+    }
+  }
+
+
+  ## Trailing cat pass
+
+  my $last_command = $commands->[-1]->{shell_arg};
+
+  if ($last_command->[0] eq 'cat' && @$last_command == 1) {
+    if (-t STDOUT) {
+      debug_log("OPT: did *NOT* optimise away trailing cat because STDOUT is a terminal");
+    } else {
+      debug_log("OPT: optimised away trailing cat");
+      pop @$commands;
+    }
+  }
+
+
+
+  $self->{compiled_commands} = $commands;
+
+  return $self;
+}
+
+
+
+
+
 
 sub execute {
   my ($self) = @_;
@@ -137,6 +214,8 @@ sub execute {
 
   #say STDERR 'reaped ' . 
   waitpid(-1, 0) . " ($?)" for 1..@$commands;
+
+  return $self;
 }
 
 
